@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2022  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -30,7 +30,7 @@ from itertools import groupby
 from collections import namedtuple
 import os.path
 import logging
-from macsypy.error import MacsypyError
+from macsypy.error import MacsypyError, EmptyFileError
 _log = logging.getLogger(__name__)
 
 
@@ -52,7 +52,14 @@ def fasta_iter(fasta_file):
         header = header.split()
         _id = header[0]
         comment = ' '.join(header[1:])
-        seq = ''.join(s.strip() for s in next(faiter))
+        try:
+            seq = ''.join(s.strip() for s in next(faiter))
+        except StopIteration:
+            # the sequence was not start by '>'
+            # bad fasta format
+            msg = f"Error during sequence '{fasta_file.name}' parsing: Check the fasta format."
+            _log.critical(msg)
+            raise MacsypyError(msg)
         length = len(seq)
         yield _id, comment, length
 
@@ -116,6 +123,12 @@ class Indexes:
                     # the separator is different than the actual separator
                     _log.warning(f"The '{my_indexes}' index file is in old format. Force index building.")
                     force = True
+            # if fasta file is newer than idx
+            stamp_fasta = os.path.getmtime(self._fasta_path)
+            stamp_idx = os.path.getmtime(my_indexes)
+            if stamp_idx < stamp_fasta:
+                _log.debug("the sequence index is older than sequence file: rebuild the index.")
+                force = True
 
         if force or not my_indexes:
             try:
@@ -125,9 +138,7 @@ class Indexes:
                 _log.critical(msg)
                 raise IOError(msg) from None
 
-            index = self._build_my_indexes(index_dir)
-            my_indexes = index
-
+            my_indexes = self._build_my_indexes(index_dir)
         return my_indexes
 
 
@@ -197,6 +208,10 @@ class Indexes:
             msg = f"unable to index the sequence dataset: {self.cfg.sequence_db()} : {err}"
             _log.critical(msg, exc_info=True)
             raise MacsypyError(msg) from err
+        if seq_nb == 0:
+            os.unlink(index_file)
+            msg = f"The sequence-db file '{self._fasta_path}' does not contains sequences."
+            raise EmptyFileError(msg)
         return index_file
 
 
@@ -218,7 +233,10 @@ class Indexes:
                 try:
                     seq_id, length, _rank = line.split(self._field_separator)
                 except Exception as err:
-                    raise MacsypyError(f"fail to parse database index {path} at line: {line}", err) from err
+                    raise MacsypyError(f"fail to parse database index {path} at line: {line}."
+                                       f"Try to rebuild index with --idx option or remove file."
+                                       f"If error persist feel free to submit an issue at"
+                                       f"https://github.com/gem-pasteur/macsyfinder/issues/new?assignees=&labels=bug&template=bug_report.md&title=%5BBUG%5D ", err) from err
                 length = int(length)
                 _rank = int(_rank)
                 yield seq_id, length, _rank
@@ -256,7 +274,6 @@ class RepliconDB:
     the Replicon object must be instantiated only for sequence_db of type 'gembase' or 'ordered_replicon'
     """
 
-    ordered_replicon_name = 'UserReplicon'
 
     def __init__(self, cfg):
         """
@@ -278,8 +295,13 @@ class RepliconDB:
         if self.cfg.db_type() == 'gembase':
             self._fill_gembase_min_max(topo_dict, default_topology=self.cfg.replicon_topology())
         else:
+            self._ordered_replicon_name = os.path.splitext(os.path.basename(self.cfg.sequence_db()))[0]
             self._fill_ordered_min_max(self.cfg.replicon_topology())
 
+    @property
+    def ordered_replicon_name(self):
+        return self._ordered_replicon_name
+
     def guess_if_really_gembase(self):
         """
         Count the number of replicon with only on sequence
@@ -354,9 +376,11 @@ class RepliconDB:
             in gembase the identifier of fasta sequence follows the following schema:
             <replicon-name>_<seq-name> with eventually '_' inside the <replicon_name>
             but not in the <seq-name>.
+            for draft genome the seqname is postfix with 'b' if the sequence is border of the contig
+            or 'i' if it is inside.
             so grp_replicon allow to group sequences belonging to the same replicon.
             """
-            return "_".join(entry[0].split('_')[: -1])
+            return "_".join(entry[0].split('_')[: -1]).rstrip('ib')
 
         def parse_seq_id(seq_id):
             """
