@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2022  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -149,10 +149,11 @@ class Profile:
         return f"{self.gene.name} : {self.path}"
 
 
-    def execute(self):
+    def execute(self, cpu=1):
         """
         Launch the Hmmer search (hmmsearch executable) with this profile
 
+        :param int cpu: the number of cpu to use for hmmsearch (must be >= 1)
         :return: an object storing information on the results of the HMM search (HMMReport)
         :rtype:  :class:`macsypy.report.HMMReport` object
         """
@@ -172,17 +173,17 @@ class Profile:
                                     self.gene.name + os.path.splitext(self.cfg.res_search_suffix())[0] + ".err")
 
             with open(err_path, 'w') as err_file:
-                if self.cfg.no_cut_ga():
+                if not self.cfg.cut_ga():
                     hmmer_threshold = f"-E {self.cfg.e_value_search():f}"
-                elif not self.cfg.no_cut_ga() and self.ga_threshold:
+                elif self.cfg.cut_ga() and self.ga_threshold:
                     hmmer_threshold = "--cut_ga"
                 else:
-                    # no_cut_ga is not set but there is not self.ga_threshold:
+                    # cut_ga is True set but there is not self.ga_threshold:
                     hmmer_threshold = f"-E {self.cfg.e_value_search():f}"
                     _log.warning(f"GA bit thresholds unavailable on profile {self.gene.name}. "
                                  f"Switch to e-value threshold ({hmmer_threshold})")
 
-                command = f'"{self.cfg.hmmer()}" --cpu 1 -o "{output_path}" {hmmer_threshold} ' \
+                command = f'"{self.cfg.hmmer()}" --cpu {cpu} -o "{output_path}" {hmmer_threshold} ' \
                           f'"{self.path}" "{self.cfg.sequence_db()}" '
                 _log.debug(f"{self.gene.name} Hmmer command line : {command}")
                 try:
