@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2022  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -25,12 +25,15 @@
 """
 Main entrypoint to macsyfinder
 """
+import shutil
 
 import sys
 import os
 import argparse
 import logging
 import itertools
+import signal
+import time
 
 from operator import attrgetter  # To be used with "sorted"
 from textwrap import dedent
@@ -47,12 +50,12 @@ from macsypy.registries import ModelRegistry, scan_models_dir
 from macsypy.definition_parser import DefinitionParser
 from macsypy.search_genes import search_genes
 from macsypy.database import Indexes, RepliconDB
-from macsypy.error import OptionError
+from macsypy.error import OptionError, Timeout, EmptyFileError
 from macsypy import cluster
 from macsypy.hit import get_best_hits, HitWeight, MultiSystem, LonerMultiSystem, \
     sort_model_hits, compute_best_MSHit
 from macsypy.system import OrderedMatchMaker, UnorderedMatchMaker, System, LikelySystem, UnlikelySystem, HitSystemTracker
-from macsypy.utils import get_def_to_detect, get_replicon_names
+from macsypy.utils import get_def_to_detect, get_replicon_names, parse_time
 from macsypy.profile import ProfileFactory
 from macsypy.model import ModelBank
 from macsypy.gene import GeneBank
@@ -61,6 +64,16 @@ from macsypy.serialization import TxtSystemSerializer, TxtLikelySystemSerializer
     TsvSystemSerializer, TsvSolutionSerializer, TsvLikelySystemSerializer, TsvSpecialHitSerializer, TsvRejectedCandidatesSerializer
 
 
+def alarm_handler(signum, frame):
+    _log.critical("Timeout is over. Aborting")
+    for h in _log.handlers:
+        h.flush()
+    # I exit wit 0 otherwise in parallel_msf the job will be retry
+    # on an other machine. we don't want that.
+    #sys.exit(0)
+    raise Timeout()
+
+
 def get_version_message():
     """
     :return: the long description of the macsyfinder version
@@ -272,15 +285,25 @@ If a profile does not contains the GA bit score the --e-value-search (-E in hmms
 To applied the --e-value-search to all profiles use the --no-cut-ga option. 
 (default: {msf_def['e_value_search']})
 """)
-    hmmer_options.add_argument('--no-cut-ga',
+    cut_ga_group = hmmer_options.add_mutually_exclusive_group()
+    cut_ga_group.add_argument('--no-cut-ga',
                                action='store_true',
-                               default=False,
+                               default=None,
                                help=f"""By default the MSF try to applied a threshold per profile by using the
 hmmer -cut-ga option. This is possible only if the GA bit score is present in the profile otherwise 
 MF switch to use the --e-value-search (-E in hmmsearch). 
 If this option is set the --e-value-search option is used for all profiles regardless the presence of 
 the a GA bit score in the profiles.
-(default: {msf_def['no_cut_ga']})""")
+(default: {not msf_def['cut_ga']})""")
+    cut_ga_group.add_argument('--cut-ga',
+                               action='store_true',
+                               default=None,
+                               help=f"""By default the MSF try to applied a threshold per profile by using the
+hmmer -cut-ga option. This is possible only if the GA bit score is present in the profile otherwise 
+MSF switch to use the --e-value-search (-E in hmmsearch). 
+But the modeler can override this default behavior to do not use cut_ga but --e-value-search instead (-E in hmmsearch).
+The user can reestablish the general MSF behavior, be sure the profiles contain the GA bit score.
+(default: {msf_def['cut_ga']})""")
 
     hmmer_options.add_argument('--i-evalue-sel',
                                action='store',
@@ -362,6 +385,14 @@ to organize the models in subsystems.""")
                              default=None,
                              help="""Path to the directory where to store output results.
 if out-dir is specified, res-search-dir will be ignored.""")
+    dir_options.add_argument('--force',
+                             dest='force_run',
+                             action='store_true',
+                             default=None,
+                             help="""force to run even the out dir already exists and is not empty.
+Use this option with caution, MSF will erase everything in out dir before to run."""
+                             )
+
     dir_options.add_argument('--index-dir',
                              action='store',
                              default=None,
@@ -437,6 +468,16 @@ Conflicts with options:
     # by developers to generate portable data set, as for example test
     # data set, which are used on many different machines (using previous-run option).
 
+    general_options.add_argument("--timeout",
+                                 action='store',
+                                 default=None,
+                                 type=parse_time,
+                                 help="""In some case msf can take a long time to find the best solution (in 'gembase' and 'ordered_replicon mode').
+The timeout is per replicon. If this step reach the timeout, the replicon is skipped (for gembase mode the analyse of other replicons continue).
+NUMBER[SUFFIX]  NUMBER seconds. SUFFIX may be 's' for seconds (the default), 'm' for minutes, 'h' for hours or 'd' for days
+for instance 1h2m3s means 1 hour 2 min 3 sec. NUMBER must be an integer.
+""")
+
     parsed_args = parser.parse_args(args)
     if parsed_args.cfg_file and parsed_args.previous_run:
         # argparse does not allow to have mutually exclusive option  in a argument group
@@ -445,6 +486,7 @@ Conflicts with options:
         parser.print_usage()
         print("macsyfinder: error: argument --previous-run: not allowed with argument --cfg-file")
         sys.exit(2)
+
     return parser, parsed_args
 
 
@@ -463,7 +505,7 @@ def search_systems(config, model_registry, models_def_to_detect, logger):
     :type config: :class:`macsypy.config.Config` object
     :param model_registry: the registry of all models
     :type model_registry: :class:`macsypy.registries.ModelRegistry` object
-    :param models_def_to_detect: the defintions to detect
+    :param models_def_to_detect: the definitions to detect
     :type models_def_to_detect: list of :class:`macsypy.registries.DefinitionLocation` objects
     :param logger: The logger use to display information to the user.
                    It must be initialized. see :func:`macsypy.init_logger`
@@ -671,7 +713,7 @@ def _search_in_unordered_replicon(hits_by_replicon, models_to_detect, logger):
     return likely_systems, rejected_hits
 
 
-def _outfile_header(models_fam_name, models_version):
+def _outfile_header(models_fam_name, models_version, skipped_replicons=None):
     """
     :return: The 2 first lines of each result file
     :rtype: str
@@ -679,10 +721,15 @@ def _outfile_header(models_fam_name, models_version):
     header = f"""# macsyfinder {macsypy.__version__}
 # models : {models_fam_name}-{models_version}
 # {' '.join(sys.argv)}"""
+    if skipped_replicons:
+        header += "\n#"
+        for rep_name in skipped_replicons:
+            header += f"\n# WARNING: The replicon '{rep_name}' has been SKIPPED. Cannot be solved before timeout."
+        header += "\n#"
     return header
 
 
-def systems_to_tsv(models_fam_name, models_version, systems, hit_system_tracker, sys_file):
+def systems_to_tsv(models_fam_name, models_version, systems, hit_system_tracker, sys_file, skipped_replicons=None):
     """
     print systems occurrences in a file in tabulated  format
 
@@ -692,9 +739,13 @@ def systems_to_tsv(models_fam_name, models_version, systems, hit_system_tracker,
     :type hit_system_tracker: :class:`macsypy.system.HitSystemTracker` object
     :param sys_file: The file where to write down the systems occurrences
     :type sys_file: file object
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
     :return: None
     """
-    print(_outfile_header(models_fam_name, models_version), file=sys_file)
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=sys_file)
+    if skipped_replicons:
+        systems = [s for s in systems if s.replicon_name not in skipped_replicons]
     if systems:
         print("# Systems found:", file=sys_file)
         print(TsvSystemSerializer.header, file=sys_file)
@@ -708,7 +759,7 @@ def systems_to_tsv(models_fam_name, models_version, systems, hit_system_tracker,
         print("# No Systems found", file=sys_file)
 
 
-def systems_to_txt(models_fam_name, models_version, systems, hit_system_tracker, sys_file):
+def systems_to_txt(models_fam_name, models_version, systems, hit_system_tracker, sys_file, skipped_replicons=None):
     """
     print systems occurrences in a file in human readable format
 
@@ -718,10 +769,14 @@ def systems_to_txt(models_fam_name, models_version, systems, hit_system_tracker,
     :type hit_system_tracker: :class:`macsypy.system.HitSystemTracker` object
     :param sys_file: The file where to write down the systems occurrences
     :type sys_file: file object
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
     :return: None
     """
 
-    print(_outfile_header(models_fam_name, models_version), file=sys_file)
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=sys_file)
+    if skipped_replicons:
+        systems = [s for s in systems if s.replicon_name not in skipped_replicons]
     if systems:
         print("# Systems found:\n", file=sys_file)
         for system in systems:
@@ -736,7 +791,7 @@ def systems_to_txt(models_fam_name, models_version, systems, hit_system_tracker,
         print("# No Systems found", file=sys_file)
 
 
-def solutions_to_tsv(models_fam_name, models_version, solutions, hit_system_tracker, sys_file):
+def solutions_to_tsv(models_fam_name, models_version, solutions, hit_system_tracker, sys_file, skipped_replicons=None):
     """
     print solution in a file in tabulated format
     A solution is a set of systems which represents an optimal combination of
@@ -748,9 +803,11 @@ def solutions_to_tsv(models_fam_name, models_version, solutions, hit_system_trac
     :type hit_system_tracker: :class:`macsypy.system.HitSystemTracker` object
     :param sys_file: The file where to write down the systems occurrences
     :type sys_file: file object
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
     :return: None
     """
-    print(_outfile_header(models_fam_name, models_version), file=sys_file)
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=sys_file)
     if solutions:
         sol_serializer = TsvSolutionSerializer()
         print("# Systems found:", file=sys_file)
@@ -794,7 +851,8 @@ def _loner_warning(systems):
     return warnings
 
 
-def summary_best_solution(models_fam_name, models_version, best_solution_path, sys_file, models_fqn, replicon_names):
+def summary_best_solution(models_fam_name, models_version, best_solution_path, sys_file, models_fqn, replicon_names,
+                          skipped_replicons=None):
     """
     do a summary of best_solution in best_solution_path and write it on out_path
     a summary compute the number of system occurrence for each model and each replicon
@@ -812,13 +870,24 @@ def summary_best_solution(models_fam_name, models_version, best_solution_path, s
     :type models_fqn: list of string
     :param replicon_names: the name of the replicons used
     :type replicon_names: list of string
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
     """
-    print(_outfile_header(models_fam_name, models_version), file=sys_file)
+    skipped_replicons = skipped_replicons if skipped_replicons else set()
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=sys_file)
 
     def fill_replicon(summary):
+        """
+        add row with 0 for all models for lacking replicons
+
+        :param summary: the
+        :type summary: :class:`pandas.DataFrame` object
+        :return:
+        :rtype: :class:`pandas.DataFrame` object
+        """
         index_name = summary.index.name
         computed_replicons = set(summary.index)
-        lacking_replicons = set(replicon_names) - computed_replicons
+        lacking_replicons = set(replicon_names) - computed_replicons - set(skipped_replicons)
         lacking_replicons = sorted(lacking_replicons)
         rows = pd.DataFrame({models: [0 * len(lacking_replicons)] for models in summary.columns}, index=lacking_replicons)
         summary = pd.concat([summary, rows], ignore_index=False)
@@ -826,6 +895,14 @@ def summary_best_solution(models_fam_name, models_version, best_solution_path, s
         return summary
 
     def fill_models(summary):
+        """
+        add columns for lacking models (it means no occurence found)
+
+        :param summary:
+        :type summary: :class:`pandas.DataFrame` object
+        :return:
+        :rtype: :class:`pandas.DataFrame` object
+        """
         computed_models = set(summary.columns)
         lacking_models = set(models_fqn) - computed_models
         lacking_models = sorted(lacking_models)
@@ -836,7 +913,13 @@ def summary_best_solution(models_fam_name, models_version, best_solution_path, s
     try:
         best_solution = pd.read_csv(best_solution_path, sep='\t', comment='#')
     except pd.errors.EmptyDataError:
-        summary = pd.DataFrame(0, index=replicon_names, columns=models_fqn)
+        # No results Found
+        # may be there is no results so I have to report
+        # may be the solution cannot be found So I do not to report (Warning)
+        # may be the both one replicon without results one replicon not solved
+        # So I have to report only the replicon without results
+        replicon_to_report = list(set(replicon_names) - set(skipped_replicons))
+        summary = pd.DataFrame(0, index=replicon_to_report, columns=models_fqn)
         summary.index.name = 'replicon'
     else:
         selection = best_solution[['replicon', 'sys_id', 'model_fqn']]
@@ -898,7 +981,8 @@ def multisystems_to_tsv(models_fam_name, models_version, systems, sys_file):
         print("# No Multisystems found", file=sys_file)
 
 
-def rejected_candidates_to_txt(models_fam_name, models_version, rejected_candidates, cand_file):
+def rejected_candidates_to_txt(models_fam_name, models_version, rejected_candidates, cand_file,
+                               skipped_replicons=None):
     """
     print rejected clusters in a file
 
@@ -906,9 +990,13 @@ def rejected_candidates_to_txt(models_fam_name, models_version, rejected_candida
     :type rejected_candidates: list of :class:`macsypy.system.RejectedCandidate` objects
     :param cand_file: The file where to write down the rejected candidates
     :type cand_file: file object
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
     :return: None
     """
-    print(_outfile_header(models_fam_name, models_version), file=cand_file)
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=cand_file)
+    if skipped_replicons:
+        rejected_candidates = [rc for rc in rejected_candidates if rc.replicon_name not in skipped_replicons]
     if rejected_candidates:
         print("# Rejected candidates:\n", file=cand_file)
         for rej_cand in rejected_candidates:
@@ -918,11 +1006,22 @@ def rejected_candidates_to_txt(models_fam_name, models_version, rejected_candida
         print("# No Rejected candidates", file=cand_file)
 
 
-def rejected_candidates_to_tsv(models_fam_name, models_version, rejected_candidates, cand_file):
+def rejected_candidates_to_tsv(models_fam_name, models_version, rejected_candidates, cand_file,
+                               skipped_replicons=None):
     """
+    print rejected clusters in a file
 
+    :param rejected_candidates: list of candidates which does not contitute a system
+    :type rejected_candidates: list of :class:`macsypy.system.RejectedCandidate` objects
+    :param cand_file: The file where to write down the rejected candidates
+    :type cand_file: file object
+    :param skipped_replicons: the replicons name for which msf reach the timeout
+    :type skipped_replicons: list of str
+    :return: None
     """
-    print(_outfile_header(models_fam_name, models_version), file=cand_file)
+    print(_outfile_header(models_fam_name, models_version, skipped_replicons=skipped_replicons), file=cand_file)
+    if skipped_replicons:
+        rejected_candidates = [rc for rc in rejected_candidates if rc.replicon_name not in skipped_replicons]
     if rejected_candidates:
         serializer = TsvRejectedCandidatesSerializer()
         rej_candidates = serializer.serialize(rejected_candidates)
@@ -1026,7 +1125,10 @@ def main(args=None, loglevel=None):
         os.makedirs(working_dir)
     else:
         if os.path.isdir(working_dir):
-            if os.listdir(working_dir):
+            if config.force_run():
+                shutil.rmtree(working_dir)
+                os.makedirs(working_dir)
+            elif os.listdir(working_dir):
                 raise ValueError(f"'{working_dir}' already exists and is not a empty")
         else:
             raise ValueError(f"'{working_dir}' already exists and is not a directory")
@@ -1087,8 +1189,14 @@ def main(args=None, loglevel=None):
         sys.exit(f"macsyfinder: {err}")
     _log.info(f"\nmodels used: {models_fam_name}-{models_version}")
     logger.info(f"\n{f' Searching systems ':#^70}")
-    all_systems, rejected_candidates = search_systems(config, model_registry, models_def_to_detect, logger)
+    try:
+        all_systems, rejected_candidates = search_systems(config, model_registry, models_def_to_detect, logger)
+    except EmptyFileError as err:
+        _log.critical(str(err))
+        sys.exit(f"macsyfinder: {err} Run Aborted.")
     track_multi_systems_hit = HitSystemTracker(all_systems)
+    skipped_replicons = []
+    
     if config.db_type() in ('gembase', 'ordered_replicon'):
         #############################
         # Ordered/Gembase replicons #
@@ -1103,13 +1211,30 @@ def main(args=None, loglevel=None):
 
         # group systems found by replicon
         # before to search best system combination
-        import time
         for rep_name, syst_group in itertools.groupby(all_systems, key=lambda s: s.replicon_name):
             syst_group = list(syst_group)
             logger.info(f"Computing best solutions for {rep_name} (nb of candidate systems {len(syst_group)})")
-            find_best_solutions_start = time.perf_counter()
-            best_sol_4_1_replicon, score = find_best_solutions(syst_group)
-            find_best_solutions_stop = time.perf_counter()
+
+            timeout = config.timeout()
+            if timeout:
+                # in some case best_solution take too much time
+                # user can define a timeout by default set to 0
+                signal.signal(signal.SIGALRM, alarm_handler)
+                signal.alarm(config.timeout())
+                _log.debug(f"set time out to {timeout} sec.")
+            try:
+                find_best_solutions_start = time.perf_counter()
+                best_sol_4_1_replicon, score = find_best_solutions(syst_group)
+                find_best_solutions_stop = time.perf_counter()
+            except Timeout:
+                _log.error(f"The {rep_name} cannot be solved in time skip it!")
+                skipped_replicons.append(rep_name)
+                continue
+            if timeout:
+                _log.debug("Cancel the time out.")
+                signal.alarm(0)
+                signal.signal(signal.SIGALRM, signal.SIG_DFL)
+
             logger.info(f"It took {find_best_solutions_stop - find_best_solutions_start:.2f}sec to find best solution"
                         f" ({score:.2f}) for replicon {rep_name}")
             # if several solutions are equivalent same number of system and score is same
@@ -1126,31 +1251,37 @@ def main(args=None, loglevel=None):
         tsv_filename = os.path.join(config.working_dir(), "all_systems.tsv")
 
         with open(system_filename, "w") as sys_file:
-            systems_to_txt(models_fam_name, models_version, all_systems, track_multi_systems_hit, sys_file)
+            systems_to_txt(models_fam_name, models_version, all_systems, track_multi_systems_hit, sys_file,
+                           skipped_replicons=skipped_replicons)
 
         with open(tsv_filename, "w") as tsv_file:
-            systems_to_tsv(models_fam_name, models_version, all_systems, track_multi_systems_hit, tsv_file)
+            systems_to_tsv(models_fam_name, models_version, all_systems, track_multi_systems_hit, tsv_file,
+                           skipped_replicons=skipped_replicons)
 
         cluster_filename = os.path.join(config.working_dir(), "rejected_candidates.txt")
         with open(cluster_filename, "w") as clst_file:
             rejected_candidates.sort(key=lambda clst: (clst.replicon_name, clst.model, clst.hits))
-            rejected_candidates_to_txt(models_fam_name, models_version, rejected_candidates, clst_file)
+            rejected_candidates_to_txt(models_fam_name, models_version, rejected_candidates, clst_file,
+                                       skipped_replicons=skipped_replicons)
         if not (all_systems or rejected_candidates):
             logger.info("No Systems found in this dataset.")
 
         cluster_filename = os.path.join(config.working_dir(), "rejected_candidates.tsv")
         with open(cluster_filename, "w") as clst_file:
-            rejected_candidates_to_tsv(models_fam_name, models_version, rejected_candidates, clst_file)
+            rejected_candidates_to_tsv(models_fam_name, models_version, rejected_candidates, clst_file,
+                                       skipped_replicons=skipped_replicons)
 
         tsv_filename = os.path.join(config.working_dir(), "all_best_solutions.tsv")
         with open(tsv_filename, "w") as tsv_file:
-            solutions_to_tsv(models_fam_name, models_version, all_best_solutions, track_multi_systems_hit, tsv_file)
+            solutions_to_tsv(models_fam_name, models_version, all_best_solutions, track_multi_systems_hit, tsv_file,
+                             skipped_replicons=skipped_replicons)
 
         best_solution_filename = os.path.join(config.working_dir(), "best_solution.tsv")
         with open(best_solution_filename, "w") as best_solution_file:
             one_best_solution = [syst for sol in one_best_solution for syst in sol]
             one_best_solution.sort(key=lambda syst: (syst.replicon_name, syst.position[0], syst.model.fqn, - syst.score))
-            systems_to_tsv(models_fam_name, models_version, one_best_solution, track_multi_systems_hit, best_solution_file)
+            systems_to_tsv(models_fam_name, models_version, one_best_solution, track_multi_systems_hit, best_solution_file,
+                           skipped_replicons=skipped_replicons)
 
         loners_filename = os.path.join(config.working_dir(), "best_solution_loners.tsv")
         with open(loners_filename, "w") as loners_file:
@@ -1163,12 +1294,9 @@ def main(args=None, loglevel=None):
         summary_filename = os.path.join(config.working_dir(), "best_solution_summary.tsv")
         with open(summary_filename, "w") as summary_file:
             models_fqn = [m.fqn for m in models_def_to_detect]
-            if config.db_type() == 'gembase':
-                replicons_names = get_replicon_names(config.sequence_db())
-            else:
-                # it's an ordered_replicon
-                replicons_names = [RepliconDB.ordered_replicon_name]
-            summary_best_solution(models_fam_name, models_version, best_solution_filename, summary_file, models_fqn, replicons_names)
+            replicons_names = get_replicon_names(config.sequence_db(), config.db_type())
+            summary_best_solution(models_fam_name, models_version, best_solution_filename, summary_file, models_fqn, replicons_names,
+                                  skipped_replicons=skipped_replicons)
 
     else:
         #######################
@@ -1199,7 +1327,9 @@ def main(args=None, loglevel=None):
 
         if not (all_systems or rejected_candidates):
             logger.info("No Systems found in this dataset.")
-
+    if skipped_replicons:
+        for rep_name in skipped_replicons:
+            _log.error(f"The replicon {rep_name} cannot be solved before timeout. SKIP IT.")
     logger.info("END")
 
 
