@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2022  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -25,17 +25,70 @@
 
 import os
 import platform
+import time
 import unittest
 import shutil
 import tempfile
 import argparse
 
 from macsypy.config import Config, MacsyDefaults
-from macsypy.database import Indexes
-from macsypy.error import MacsypyError
+from macsypy.database import Indexes, fasta_iter
+from macsypy.error import MacsypyError, EmptyFileError
 from tests import MacsyTest
 
 
+class TestFastaIter(MacsyTest):
+
+    def setUp(self) -> None:
+        self.sequences = [
+            ('seq1', 'comment 1', 'ATGCATGC' ),
+            ('seq2', 'comment 2', 'GGGGCCCCTT')
+        ]
+        self.tmpdir = os.path.join(tempfile.gettempdir(), 'test_macsyfinder_fasta_iter')
+        if os.path.exists(self.tmpdir):
+            shutil.rmtree(self.tmpdir)
+        os.makedirs(self.tmpdir)
+
+    def tearDown(self):
+        try:
+            shutil.rmtree(self.tmpdir)
+        except:
+            pass
+
+    def test_fasta_iter(self):
+        fasta_path = os.path.join(self.tmpdir, "sequence.fa")
+        with open(fasta_path, 'w') as fasta:
+            for id_, comment, seq in self.sequences:
+                fasta.write(f">{id_} {comment}\n{seq}\n")
+
+        with open(fasta_path) as fasta:
+            f_iter = fasta_iter(fasta)
+            for seq_read, seq_expected in zip(f_iter, self.sequences):
+                self.assertEqual(seq_read[0], seq_expected[0])
+                self.assertEqual(seq_read[1], seq_expected[1])
+                self.assertEqual(seq_read[2], len(seq_expected[2]))
+
+    def test_fasta_iter_bad_fasta(self):
+        fasta_path = os.path.join(self.tmpdir, "sequence.fa")
+        with open(fasta_path, 'w') as fasta:
+            first = True
+            for id_, comment, seq in self.sequences:
+                if first:
+                    fasta.write(f" >{id_} {comment}\n{seq}\n")
+                    first = False
+                else:
+                    fasta.write(f">{id_} {comment}\n{seq}\n")
+
+        with open(fasta_path) as fasta:
+            f_iter = fasta_iter(fasta)
+            with self.assertRaises(MacsypyError) as ctx:
+                with self.catch_log():
+                    for seq in f_iter:
+                        pass
+            self.assertEqual(str(ctx.exception),
+                             f"Error during sequence '{fasta_path}' parsing: Check the fasta format.")
+
+
 class TestIndex(MacsyTest):
 
     def setUp(self):
@@ -44,7 +97,7 @@ class TestIndex(MacsyTest):
         args.models_dir = self.find_data('models')
         args.out_dir = os.path.join(tempfile.gettempdir(), 'test_macsyfinder_indexes')
         if os.path.exists(args.out_dir):
-            shutil.rmtree(os.path.join(tempfile.gettempdir(), 'test_macsyfinder_indexes'))
+            shutil.rmtree(args.out_dir)
         os.makedirs(args.out_dir)
         seq_db = self.find_data("base", "test_1.fasta")
         shutil.copy(seq_db, args.out_dir)
@@ -72,9 +125,52 @@ class TestIndex(MacsyTest):
 
     def test_build_no_idx(self):
         idx = Indexes(self.cfg)
-        my_idx =idx.build()
+        my_idx = idx.build()
         self.assertEqual(my_idx, os.path.join(os.path.dirname(self.cfg.sequence_db()), idx.name + ".idx"))
 
+    def test_build_idx_older_than_fasta(self):
+        # test if idx.build
+        # if index is present and newer than fasta the index are not rebuild
+        # if index is present but older than fasta the index is rebuild
+        fasta_path = self.cfg.sequence_db()
+        idx = Indexes(self.cfg)
+        idx_path = idx.build()
+        first_build_stamp = os.path.getmtime(idx_path)
+        idx_path = idx.build()
+        second_build_stamp = os.path.getmtime(idx_path)
+        time.sleep(.2)
+        f = open(fasta_path, 'a')
+        f.write('\n')
+        f.close()
+        idx_path = idx.build()
+        third_build_stamp = os.path.getmtime(idx_path)
+        self.assertEqual(first_build_stamp, second_build_stamp)
+        self.assertGreater(third_build_stamp, second_build_stamp)
+
+    def test_build_idx_point_wrong_fasta(self):
+        # test if idx.build
+        # if index is present and newer than fasta the index are not rebuild
+        # if index is present but older than fasta the index is rebuild
+        fasta_path = self.cfg.sequence_db()
+        idx = Indexes(self.cfg)
+        idx_path = idx.build()
+        first_build_stamp = os.path.getmtime(idx_path)
+        with open(idx_path, 'r') as f:
+            idx_content = f.readlines()
+        first_header = idx_content[0]
+        bad_header = f"{first_header[:-1]}_fake\n"
+        idx_content[0] = bad_header
+        time.sleep(.2)
+        with open(idx_path, 'w') as f:
+            f.write(''.join(idx_content))
+        with self.catch_log():
+            idx_path = idx.build()
+        second_build_stamp = os.path.getmtime(idx_path)
+        self.assertGreater(second_build_stamp, first_build_stamp)
+        with open(idx_path, 'r') as f:
+            new_header = f.readline()
+        self.assertEqual(new_header, first_header)
+
 
     def test_build_with_idx(self):
         idx = Indexes(self.cfg)
@@ -319,3 +415,26 @@ VICH001.B.00001.C001_01565{idx._field_separator}414{idx._field_separator}49
                         ('VICH001.B.00001.C001_01565', 414, 49)]
         self.assertListEqual(list(iter(idx)), expected_idx)
 
+
+    def test_empty_fasta(self):
+        args = argparse.Namespace()
+        args.models_dir = self.find_data('models')
+        args.out_dir = os.path.join(tempfile.gettempdir(), 'test_macsyfinder_indexes')
+        args.index_dir = args.out_dir
+
+        seq_db = "empty.fasta"
+        open(os.path.join(args.out_dir, seq_db), 'w').close()
+        args.sequence_db = os.path.join(args.out_dir, seq_db)
+
+        for db_type in 'gembase', 'ordered_replicon', 'unordered':
+            with self.subTest(db_type=db_type):
+                args.db_type = db_type
+                cfg = Config(MacsyDefaults(), args)
+                idx = Indexes(cfg)
+                with self.assertRaises(EmptyFileError) as ctx:
+                    idx.build()
+                self.assertEqual(str(ctx.exception),
+                                 f"The sequence-db file '{args.sequence_db}' does not contains sequences.")
+
+                self.assertFalse(os.path.exists(f"{args.sequence_db}.idx"))
+
